/*
 * Copyright 2023 Hartmut Seichter
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
#ifndef PARADISO_GEOMETRY_HPP
#define PARADISO_GEOMETRY_HPP

#include <paradiso/globals.hpp>

namespace paradiso
{

    struct Point final
    {
        using value_type = int32_t;
        value_type x{0}, y{0};
    };

    struct Size final
    {
        using value_type = uint32_t;
        value_type width{0}, height{0};

        constexpr auto area() const noexcept
        {
            return width * height;
        }
    };

    struct Rectangle
    {
        Point position;
        Size size;

        constexpr bool contains(const Point &p) const noexcept
        {
            return p.x >= position.x && p.x <= position.x + size.width &&
                   p.y >= position.y && p.y <= position.y + size.height;
        }
    };
}

#endif