/*
 * Copyright 2023 Hartmut Seichter
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
#ifndef PARADISO_SPRITE_HPP
#define PARADISO_SPRITE_HPP

#include <paradiso/globals.hpp>
#include <paradiso/bitmap.hpp>
#include <paradiso/matrix.hpp>
#include <paradiso/vector.hpp>

namespace paradiso {

struct Sprite final {
    using ChangeCountType = std::uint64_t;

    static constexpr Sprite create() noexcept { return {}; }

    Bitmap bitmap{};

    Vector2<float> pivot{Vector2<float>::zero()};

    std::array<std::uint32_t, 6> indices{0, 3, 2, 2, 1, 0};

    std::array<Vector3<float>, 4> vertices{
        Vector3<float>::make(-1.0f, -1.0f, 0.0f),
        Vector3<float>::make(-1.0f, +1.0f, 0.0f),
        Vector3<float>::make(+1.0f, +1.0f, 0.0f),
        Vector3<float>::make(+1.0f, -1.0f, 0.0f)};

    std::array<Vector3<float>, 4> normals{
        Vector3<float>::z_axis(), Vector3<float>::z_axis(),
        Vector3<float>::z_axis(), Vector3<float>::z_axis()};

    std::array<Vector2<float>, 4> texture_coordinates{
        Vector2<float>::make(0.0f, 0.0f), /// 0
        Vector2<float>::make(0.0f, 1.0f), /// 1
        Vector2<float>::make(1.0f, 1.0f), /// 2
        Vector2<float>::make(1.0f, 0.0f)  /// 3
    };

    ChangeCountType change_count{};

};
} // namespace paradiso

#endif